//-------------------------------------------------------------------------------------------------
// CSP2.H
//
// Terminal: OPN2001 
// 
// Description: API interface for the communication protocol used in the OPN-2001
//

#ifndef __CSP2_H__
#define __CSP2_H__

// use this to make this file also work with normal c-compilers
#ifdef __cplusplus
extern "C"
{
#endif

#ifdef CSP2_EXPORTS
	#define CSP2_API __declspec(dllexport) __stdcall
#else
	#define CSP2_API __declspec(dllimport) __stdcall
#endif

//
// Returned status values...
//
#define STATUS_OK                   (0L)
#define COMMUNICATIONS_ERROR        (-1L)  
#define BAD_PARAM                   (-2L)
#define SETUP_ERROR                 (-3L)
#define INVALID_COMMAND_NUMBER      (-4L)  
#define COMMAND_LRC_ERROR           (-7L)  
#define RECEIVED_CHARACTER_ERROR    (-8L)  
#define GENERAL_ERROR               (-9L)  
#define FILE_NOT_FOUND              (2L)
#define ACCESS_DENIED               (5L)

// Parameter values...
#define PARAM_OFF                   (0L) 
#define PARAM_ON                    (1L) 

#define DATA_AVAILABLE              (1L)
#define NO_DATA_AVAILABLE           (0L)

#define DETERMINE_SIZE              (0L)

//
// Strange way of selecting serial ports
// Should only be used with the csp2Init() function
// 
typedef enum __serial_ports{
	COM1 = 0,
	COM2,
	COM3,
	COM4,
	COM5,
	COM6,
	COM7,
	COM8,
	COM9,
	COM10,
	COM11,
	COM12,
	COM13,
	COM14,
	COM15,
	COM16
}_serial_ports;

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2Init( long nCOMPort );
//
// Description:		Open the COM port with one of the _serial_ports enum
//					The values that can be used are from COM1 .. COM16
//					Do not use normal numbers
//
// Argument:	nCOMPort the constant _serial_ports number from COM1 .. COM16
//
// Important:		COM1 is used when nComport == 0 see _serial_ports enum,
//					COM16 is the highest port number that can be used, when higher ports are 
//					needed use the csp2InitEx() function.
//
// Returns:			0L		on success 
//					< 0L	on failure 
//
long CSP2_API csp2Init( long nCOMPort );	// important COM1 is initializes when nCOMPort = 0 supports up to COM16

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2InitEx( long nCOMPort );
//
// Description:	Open the COM port. This functions supports from COM1 .. COM255.
//				Do not use the constants from the _serial_ports enum, but use 
//				1L for COM1, 2 for COM2 etc..
//
// Argument:	nCOMPort serial port number
//
// Important:	COM1 is used when nCOMPort == 1
//
// Returns:		0L		on success 
//				< 0L	on failure 
//
long CSP2_API csp2InitEx( long nCOMPort );	// important COM1 is initializes when nComport = 1 supports all available COM ports

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2Restore( void );
//
// Description:	Closes the COM port that was opened by the csp2Init or csp2InitEx
//
// Returns:		0L		on success 
//				< 0L	on failure 
//
long CSP2_API csp2Restore( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2Wakeup( void );
//
// Description:	Set the DTR (data-terminal-ready) signal. This function is added 
//				for compatibility reasons.
//
// Returns:		0L		on success 
//				< 0L	on failure 
//
long CSP2_API csp2WakeUp( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2DataAvailable( void );
//
// Description:	Checks if CTS (clear-to-send) signal is ON. When the CTS line is ON
//				Data is available in the OPN-2001
//
// Returns:		> 0L		Data available
//				0L			No data available
//				< 0L		on failure 
//
long CSP2_API csp2DataAvailable( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2ReadData( void );
//
// Description:	Read all data from the OPN-2001 and store it in an 
//				internal buffer. To read data from the internal buffer use 
//				one of the functions below.
//					* csp2GetPacket()
//					* csp2GetDeviceId()
//
// Returns:		>= 0L		The amount of barcodes that are collected
//				< 0L		on failure 
long CSP2_API csp2ReadData( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2ClearData( void );
//
// Description:	Removes all collected barcodes from the OPN-2001
//
// Returns:		0L		on success
//				< 0L	on failure 
//
long CSP2_API csp2ClearData( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2PowerDown( void );
//
// Description:	Send a power down command to the OPN-2001.
//				This function was added for compatibility reasons only.
//				
// Returns:		0L		on success
//				< 0L	on failure 
//
long CSP2_API csp2PowerDown( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetTime( unsigned char ucTime[] );
//
// Description:	Get the current time from the OPN-2001.
//
// Argument:	ucTime needs to be character buffer of at least 6 bytes
//				ucTime[ 0 ] = Seconds
//				ucTime[ 1 ] = Minutes
//				ucTime[ 2 ] = Hours
//				ucTime[ 3 ] = Day
//				ucTime[ 4 ] = Month
//				ucTime[ 5 ] = Year
//				
// Returns:		0L		on success
//				< 0L	on failure 
//
long CSP2_API csp2GetTime( unsigned char ucTime[] );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2SetTime( unsigned char ucTime[] );
//
// Description:	Set a new time and date in the OPN-2001
//
// Argument:	ucTime needs to be a character buffer of at least 6 bytes
//				ucTime[ 0 ] = Seconds
//				ucTime[ 1 ] = Minutes
//				ucTime[ 2 ] = Hours
//				ucTime[ 3 ] = Day
//				ucTime[ 4 ] = Month
//				ucTime[ 5 ] = Year
//				
// Returns:		0L		on success
//				< 0L	on failure 
//
long CSP2_API csp2SetTime( unsigned char ucTime[] );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2SetDefault( void );
//
// Description:	Set the OPN-2001 into default mode
//				
// Returns:		0L		on success
//				< 0L	on failure 
//
long CSP2_API csp2SetDefaults( void );


//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetPacket( char szData[], long nBcrNr, long nMaxDataSz );
//
// Description:	Get data that was stored in the internal buffer by the csp2ReadData function.
//
// Argument:	szData[] buffer where the barcode data is stored in
//				nBcrNr the wanted barcode number starting from 0L
//				nMaxDataSz the maximum size of the szData buffer
//				
// Returns:		0L		on success
//				< 0L	on failure 
//
long CSP2_API csp2GetPacket( char szData[], long nBcrNr, long nMaxDataSz );


//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetDeviceId( char cDeviceId[8], long nMaxDeviceIdSz );
//
// Description:	Get the device ID number from the OPN-2001
//
// Parameters:	cDeviceId[8] is the buffer where the device ID is copied into
//				nMaxDeviceIdSz is the maximum size of the szDeviceId buffer
//			
// Important:	In the cDeviceId buffer an Hex respresentation of the Device ID is placed
//
// Returns:		0L	0n success
//				< 0L		on failure 
//
long CSP2_API csp2GetDeviceId( char cDeviceId[8], long nMaxDeviceIdSz );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetProtocol( void );
//
// Description:	Get the OPN-2001 protocol number. 
//				
// Returns:		>= 0L	Protocol version number
//				< 0L	on failure 
//
long CSP2_API csp2GetProtocol( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetSystemStatus( void );
//
// Description:	Get the systemstatus of the OPN-2001.
//				
// Returns:		= STATUS_OK		System OK
//				= 22L			Low battery
//				< 0L			on failure 
//
long CSP2_API csp2GetSystemStatus( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetSwVersion( char szSwVersion[9], long nMaxSwVersionSz );
//
// Description:	Get the software version string from OPN-2001
//
// Argument:	szSwVersion[9] buffer where the software version is copied into
//				nMaxLength the maximum size of the szSwVersion
//				
// Returns:		>= 0L		The needed buffer size to collect all version data including the '\0' character
//				< 0L		on failure 
//
long CSP2_API csp2GetSwVersion( char szSwVersion[9], long nMaxSwVersionSz );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetASCIIMode( void );
//
// Description:	Returns a value telling if the OPN-2001 is in ASCII mode
//				
// Returns:		= 1L		ASCII mode ON
//				= 0L		ASCII mode OFF
//				< 0L		on failure 
//
long CSP2_API csp2GetASCIIMode( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetRTCMode( void );
//
// Description:	Return value tells if the OPN-2001 adds a timestamp to the 
//				collected barcode data.
//				
// Returns:		= 1L		Timstamp append to barcode data
//				= 0L		No timestamp is used
//				< 0L		on failure 
//
long CSP2_API csp2GetRTCMode( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2SetRetryCount( long nRetries );
//
// Description:	Set the number of interrogation retries 
//
// Argument:	nRetries specifies the number interrogation retries
//				values from 1L .. 9L are supported
//				
// Returns:		STATUS_OK	on success
//				< 0L		on failure 
//
long CSP2_API csp2SetRetryCount( long nRetries );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetRetryCount( void );
//
// Description:	Get the number of communication retries 
//
// Returns:		1 .. 9 the number of retries
long CSP2_API csp2GetRetryCount( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetDllVersion( char szDllVersion[], long nMaxDllVersionSz );
//
// Description:	Get the software version of the DLL
//
// Argument:	szDllVersion[] buffer where the software version is copied into
//				nMaxDllVersionSz the maximum size of the szDllVersion
//				
// Returns:		STATUS_OK		on success
//				FILE_NOT_FOUND	on failure 
//
long CSP2_API csp2GetDllVersion( char szDllVersion[], long nMaxDllVersionSz );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2TimeStamp2Str( unsigned char *ucTimeStamp, char *szTimeStampResult, long nMaxTimeStampResult );
//
// Description:	convert the packed ucTimeStamp array to a normal 'timestamp' string this implementation has some 
//				problems but is needed for compatibility reasons
//
// Argument:	ucTimeStamp pointer to the packed 4 byte timestamp
//				szTimeStampResult string representation of the packed ucTimeStamp array
//				nMaxTimeStampResult the maximum size of the szTimeStampResult buffer
//
// Important:	The szTimeStampResult buffer needs to be at leat 21 bytes big.
//				time format: HH:MM:SS AM MM/DD/YYYY  
//				
// Returns:		STATUS_OK	on success
//				< 0L		on failure 
//
long CSP2_API csp2TimeStamp2Str( unsigned char *ucTimeStamp, char *szTimeStampResult, long nMaxTimeStampResult );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2TimeStamp2StrEx( unsigned char *ucTimeStamp, char *szTimeStampResult, long nMaxTimeStampResult );
//
// Description:	convert the packed ucTimeStamp array to a normal 'timestamp' string this is the correct implementation
//
// Argument:	ucTimeStamp pointer to the packed 4 byte timestamp
//				szTimeStampResult string representation of the packed ucTimeStamp array
//				nMaxTimeStampResult the maximum size of the szTimeStampResult buffer
//
// Important:	The szTimeStampResult buffer needs to be at leat 21 bytes big.
//				time format: HH:MM:SS AM MM/DD/YYYY  
//				
// Returns:		STATUS_OK	on success
//				< 0L		on failure 
//
long CSP2_API csp2TimeStamp2StrEx( unsigned char *ucTimeStamp, char *szTimeStampResult, long nMaxTimeStampResult );


//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetCodeType( unsigned long nBcrCodeID, char *szCodeTypeBuf, long nMaxCodeTypeBuf );
//
// Description:	convert the OPN-2001 barcode code id to a readable format
//
// Argument:	nBcrCodeID integer value of the code ID
//				szCodeTypeBuf pointer to the result buffer
//				nMaxCodeTypeBuf the maximum size of the szCodeTypeBuf buffer
//				
// Returns:		STATUS_OK	on success
//				< 0L		on failure 
//
long CSP2_API csp2GetCodeType( unsigned long nBcrCodeID, char *szCodeTypeBuf, long nMaxCodeTypeBuf );


//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2ReadRawData( char aResultBuf[], long nMaxResultBuf );
//
// Description:	Read data from the OPN-2001 and store it in aResultBuf until 
//				nMaxResultBuf bytes are read.
//				When nMaxResultBuf == 0L the csp2ReadRawData will return the 
//				size of the result buffer needed to collect all the data from 
//				the OPN-2001. 
//
// Argument:	aResultBuf result buffer where the readed data is copied into.
//				nMaxResultBuf the maximum size of the aResultBuf
//
// Returns:		>= 0L		The needed buffer size to collect all data
//				< 0L		on failure 
//
long CSP2_API csp2ReadRawData( char aResultBuf[], long nMaxResultBuf );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2SetParam(long nParamNr, char szNewParam[], long nMaxNewParam);
//
// Description:	Write one individual device parameter at one time
//
// Parameters:	nParamNr the parameter number to change
//				szNewParam hold the new parameter setting
//				nMaxNewParam  specifies the amount of characters in szNewParam that 
//				makeup the new parameter settings
//
// Returns:		STATUS_OK	On success
//				< 0L		on failure 
long CSP2_API csp2SetParam(long nParamNr, char szNewParam[], long nMaxNewParam );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetParam(long nParamNr, char szResultParam[], long nMaxResultParam);
//
// Description:	Read one individual device parameter at one time
//
// Argument:	nParamNr the parameter number to read
//				szResultParam pointer to the buffer that contains the readed parameter setting
//				nMaxLength specifies maximum size of szResultParam can contain
//
// Returns:		STATUS_OK	On success
//				< 0L		on failure 
//
long CSP2_API csp2GetParam(long nParamNr, char szResultParam[], long nMaxResultParam );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2Interrogate( void );
//
// Description:	Request a response of the OPN-2001, this fill also the internal buffer for
//				software version, protocol version and system status.
//					* csp2GetSwVersion() 
//					* csp2GetProtocol()
//					* csp2GetSystemStatus()
//
// Returns:		STATUS_OK	on success
//				< 0L		on failure
long CSP2_API csp2Interrogate( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2GetCTS( void );
//
// Description:	returns the CTS (clear-to-send) signal status.
//
// Returns:		1L			ON
//				0L			OFF
//				< 0L		on failure
//
long CSP2_API csp2GetCTS( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2SetDTR( long nOn );
//
// Description:	Set the DTR (data-terminal-ready) flow control line status
//
// Argument:	nOn = 1L set DTR line ON (high), nOn = 0L clears DTR line to OFF (low)
//
// Returns:		STATUS_OK	on success
//				< 0L		on failure
//
long CSP2_API csp2SetDTR( long nOn );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2SetDebugMode( long nOn );
//
// Description:	Sets the debug version of the CSP2.DLL in debug mode.
//
// Argument:	nOnO 1L == ON, 0L == OFF
//
// Important:	Works only with the debug version of the CSP2.DLL
//
// Returns:		STATUS_OK	on success
//				< 0L		on failure
//
long CSP2_API csp2SetDebugMode( long nOn );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2StartPolling( FARPROC csp2CallBack );
//
// Description:	This function creates a new thread to poll for OPN-2001.
//				The csp2CallBack function address is called by the thread to 
//				notify that a OPN-2001 is detected.
//
// Argument:	csp2CallBack address of the callback function
//				format of callback function int __stdcall csp2PollCallBack( void )
//
// Returns:		STATUS_OK	on success
//				< 0L		on failure
//
long CSP2_API csp2StartPolling( FARPROC csp2CallBack );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2StopPolling( void );
//
// Description:	Stops the polling thread and remove it from memory
//
// Returns:		STATUS_OK	on success
//				< 0L		on failure
long CSP2_API csp2StopPolling( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2EnablePolling( void );
//
// Description:	Signals the polling thread to resume operation.
//
// Returns:		STATUS_OK	on success
//				< 0L		on failure
//
long CSP2_API csp2EnablePolling( void );

//-------------------------------------------------------------------------------------------------
// long CSP2_API csp2DisablePolling( void );
//
// Description:	Signals the polling thread  to into suspend mode 
//
// Returns:		STATUS_OK	on success
//				< 0L		on failure
long CSP2_API csp2DisablePolling( void );

//-------------------------------------------------------------------------------------------------
// Obsolete functions 
// 
// The functions below are obsolete and should therefore not be used
// The declaration of these function have been added for compatibility reasons only.
//
long CSP2_API csp2GetCommInfo( long nCOMPort );
long CSP2_API csp2SetMultiParam( char szMultiParams[], long nMaxMultiParams );

#ifdef __cplusplus
}
#endif	// ifdef __cplusplus 


#endif // __CSP2_H__
